@@ -21,26 +21,18 @@ let common_event_message_names = [
     'timeStamp'
 ]
 
-let mouse_standard_event_message_names = [
-    'button',
-    'buttons',
+// MouseEvent and Touch events have some properties in common, so we
+// group those together.
+let mouse_and_touch_common_event_message_names = [
     'clientX',
     'clientY',
-    'layerX',
-    'layerY',
-    'movementX',
-    'movementY',
-    'offsetX',
-    'offsetY',
     'pageX',
     'pageY',
     'screenX',
     'screenY',
-    'x',
-    'y'
 ]
 
-let mouse_added_event_message_names = [
+let mouse_and_touch_added_event_message_names = [
     'dataX',
     'dataY',
     'relativeX',
@@ -56,6 +48,22 @@ let mouse_added_event_message_names = [
     'arrayY'
 ]
 
+// In addition to the common properties above, mouse events have these
+// properties that Touch events do not have.
+let mouse_standard_event_message_names = [
+    'button',
+    'buttons',
+    'layerX',
+    'layerY',
+    'movementX',
+    'movementY',
+    'offsetX',
+    'offsetY',
+    'x',
+    'y'
+]
+
+// WheelEvent also has some properties specific to it
 let wheel_standard_event_names = [
     'deltaX',
     'deltaY',
@@ -63,6 +71,20 @@ let wheel_standard_event_names = [
     'deltaMode'
 ]
 
+// For the TouchEvent object -- each of these is a list
+// of Touch events.
+let touch_event_standard_event_names = [
+    'changedTouches',
+    'targetTouches',
+    'touches'
+]
+
+// Touch also has a couple of properties specific to that kind
+// of event
+let touch_standard_event_names = [
+    'identifier'
+]
+
 let drag_standard_event_names = [
     'dataTransfer'
 ]
@@ -74,6 +96,12 @@ let key_standard_event_names = [
     'repeat'
 ]
 
+let target_property_names = [
+    'tagName',
+    'id',
+    'className',
+]
+
 let listener_cache = {}
 
 function _get_position(view, event) {
@@ -113,6 +141,7 @@ class EventModel extends WidgetModel {
             throttle_or_debounce: null,
             _supported_mouse_events: [],
             _supported_key_events: [],
+            _supported_touch_events: [],
             _modifier_keys: ['Shift', 'Control', 'Alt', 'Meta']
         });
     }
@@ -127,13 +156,16 @@ class EventModel extends WidgetModel {
         this.prepare_source()
     }
 
-    key_or_mouse(event_type) {
+    key_mouse_or_touch(event_type) {
         if (_.contains(this.get('_supported_mouse_events'), event_type)) {
             return 'mouse'
         }
         if (_.contains(this.get('_supported_key_events'), event_type)) {
             return 'keyboard'
         }
+        if (_.contains(this.get('_supported_touch_events'), event_type)) {
+            return 'touch'
+        }
     }
 
     _cache_listeners(event_type, view, handler) {
@@ -200,10 +232,11 @@ class EventModel extends WidgetModel {
     _add_listeners_to_view(view) {
         // Add listeners for each of the watched events
         for (let event of this.get('watched_events')) {
-            switch (this.key_or_mouse(event)) {
+            switch (this.key_mouse_or_touch(event)) {
                 case "keyboard":
                     this._add_key_listener(event, view)
                     break
+                case "touch":
                 case "mouse":
                     let main_event_handler = this._throttle_or_debounce(this._dom_click.bind(this, view))
                     let event_propagation_handler = this._prevent_event_propagation.bind(this, view)
@@ -256,7 +289,6 @@ class EventModel extends WidgetModel {
 
         // The actual handler is here.
         let key_handler = (event) => {
-            // console.log('Key presses FTW!', event)
             if (this.get('ignore_modifier_key_events') &&
                _.contains(this.get('_modifier_keys'), event.key)) {
                 // If the key event is supposed to be ignored, then skip it.
@@ -336,6 +368,22 @@ class EventModel extends WidgetModel {
         this._cache_listeners('mouseleave', view, disable_key_listen)
     }
 
+    _supplement_mouse_or_touch_positions(generating_view, event) {
+        // Event may be a single event, like a mouse click or wheel event, or a
+        // TouchEvent, which has several lists of Touch each of which needs
+        // supplementing.
+        if (this.key_mouse_or_touch(event.type) == 'touch') {
+            for (let touch_list of touch_event_standard_event_names) {
+                for (let touch of event[touch_list]) {
+                    this._supplement_mouse_positions(generating_view, touch)
+                }
+            }
+
+        } else {
+            this._supplement_mouse_positions(generating_view, event)
+        }
+    }
+
     _supplement_mouse_positions(generating_view, event) {
 
         // Get coordinates relative to the container
@@ -381,16 +429,16 @@ class EventModel extends WidgetModel {
     _dom_click(generating_view, event) {
         // Get coordinates relative to the container, and
         // data (i.e. "natural") coordinates.
-        this._supplement_mouse_positions(generating_view, event)
+        this._supplement_mouse_or_touch_positions(generating_view, event)
 
         // Send message to the kernel
         this._send_dom_event(event)
     }
 
     _set_xy(generating_view, event) {
-                // Get coordinates relative to the container, and
+        // Get coordinates relative to the container, and
         // data (i.e. "natural") coordinates.
-        this._supplement_mouse_positions(generating_view, event)
+        this._supplement_mouse_or_touch_positions(generating_view, event)
         let coord_type = this.get('xy_coordinate_system')
         let coords = [event[coord_type + 'X'], event[coord_type + 'Y']]
         if (coords[0] === undefined) {
@@ -408,12 +456,16 @@ class EventModel extends WidgetModel {
         // determined by the type of event from the list of event names above.
         //
         // Values are drawn from the DOM event object.
-        let event_message = {}
+        let event_message = {target: {}}
         let message_names = []
-        switch (this.key_or_mouse(event.type)) {
+        switch (this.key_mouse_or_touch(event.type)) {
+            case "touch":
+                message_names = common_event_message_names.concat(touch_event_standard_event_names)
+                break
             case "mouse":
                 message_names = common_event_message_names.concat(mouse_standard_event_message_names)
-                message_names = message_names.concat(mouse_added_event_message_names)
+                message_names = message_names.concat(mouse_and_touch_common_event_message_names)
+                message_names = message_names.concat(mouse_and_touch_added_event_message_names)
                 if (event.type == 'wheel') {
                     message_names = message_names.concat(wheel_standard_event_names)
                 } else if (event.type == 'drop' || event.type.startsWith('drag')) {
@@ -429,12 +481,40 @@ class EventModel extends WidgetModel {
         }
 
         for (let i of message_names) {
-            event_message[i] = event[i]
+            if (touch_event_standard_event_names.includes(i)) {
+                // Some TouchEvent properties are lists of Touch events.
+                // Those individual Touch events are the things that need
+                // position attributes added to them, not the top-level
+                // TouchEvent.
+                event_message[i] = this._populate_touch_event_list(event[i])
+            } else {
+                event_message[i] = event[i]
+            }
+        }
+        for (let i of target_property_names) {
+            event_message["target"][i] = event.target[i]
         }
         event_message['event'] = event['type']
         this.send(event_message, {})
     }
 
+    _populate_touch_event_list(touch_list) {
+        // Input is a touch list, return is a list with some extra properties
+        // added to each Touch position.
+        let touch_message = {}
+        let message_list = []
+        let touch_items = touch_standard_event_names.concat(mouse_and_touch_common_event_message_names)
+        touch_items = touch_items.concat(mouse_and_touch_added_event_message_names)
+
+        for (let touch of touch_list) {
+            for (let item of touch_items) {
+                touch_message[item] = touch[item]
+            }
+            message_list = message_list.concat(touch_message)
+        }
+        return message_list
+    }
+
     _prevent_event_propagation(generating_view, event) {
         // This was once in _dom_click, which handles mouse events. However, when
         // throttling or debouncing, event propagataion won't be stopped if
